import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/src/repository/location.dart';

///
/// Events
///
abstract class LocationCurrentEvent {}

class LocationCurrentRefreshEvent extends LocationCurrentEvent {}

abstract class LocationCurrentState {}

class LocationCurrentIdleState extends LocationCurrentState {}

class LocationCurrentLoadingState extends LocationCurrentState {}

class LocationCurrentLoadedState extends LocationCurrentState {
  final LatLng currentLocation;
  LocationCurrentLoadedState(this.currentLocation);
}

class LocationCurrentErrorState extends LocationCurrentState {
  final String error;

  LocationCurrentErrorState(this.error);
}

class LocationCurrentBloc
    extends Bloc<LocationCurrentEvent, LocationCurrentState> {
  final LocationRepository locationRepository;
  LocationCurrentBloc(this.locationRepository)
      : super(LocationCurrentIdleState());

  @override
  Stream<LocationCurrentState> mapEventToState(
      LocationCurrentEvent event) async* {
    if (event is LocationCurrentRefreshEvent) {
      yield LocationCurrentLoadingState();
      AppResponse<LatLng> response =
          await locationRepository.getCurrentLocation();
      if (response.isSuccess) {
        yield LocationCurrentLoadedState(response.data);
      } else {
        yield LocationCurrentErrorState(response.error);
      }
    }
  }
}
