import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/src/models/address.dart';
import 'package:ocg_location/src/repository/location.dart';

///
/// Events
///
abstract class LocationDetailEvent {}

class LocationDetailPositionEvent extends LocationDetailEvent {
  final LatLng position;

  LocationDetailPositionEvent(this.position);
}

class LocationDetailPlaceEvent extends LocationDetailEvent {
  final String placeId;

  LocationDetailPlaceEvent(this.placeId);
}

abstract class LocationDetailState {}

class LocationDetailIdleState extends LocationDetailState {}

class LocationDetailLoadingState extends LocationDetailState {}

class LocationDetailLoadedState extends LocationDetailState {
  final Address location;
  LocationDetailLoadedState(this.location);
}

class LocationDetailErrorState extends LocationDetailState {
  final String error;

  LocationDetailErrorState(this.error);
}

class LocationDetailBloc
    extends Bloc<LocationDetailEvent, LocationDetailState> {
  final LocationRepository locationRepository;
  LocationDetailBloc(this.locationRepository)
      : super(LocationDetailIdleState());

  @override
  Stream<LocationDetailState> mapEventToState(
      LocationDetailEvent event) async* {
    if (event is LocationDetailPositionEvent) {
      yield LocationDetailLoadingState();
      AppResponse<Address> response =
          await locationRepository.getLocationDetailsByPosition(event.position);
      if (response.isSuccess) {
        yield LocationDetailLoadedState(response.data);
      } else {
        yield LocationDetailErrorState(response.error);
      }
    } else if (event is LocationDetailPlaceEvent) {
      yield LocationDetailLoadingState();
      AppResponse<Address> response =
          await locationRepository.getLocationDetails(event.placeId);
      if (response.isSuccess) {
        yield LocationDetailLoadedState(response.data);
      } else {
        yield LocationDetailErrorState(response.error);
      }
    }
  }
}
