import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/src/models/address.dart';
import 'package:ocg_location/src/repository/location.dart';

///
/// Events
///
abstract class LocationSearchEvent {}

class LocationSearchLoadEvent extends LocationSearchEvent {
  final String query;

  LocationSearchLoadEvent(this.query);
}

abstract class LocationSearchState {}

class LocationSearchIdleState extends LocationSearchState {}

class LocationSearchLoadingState extends LocationSearchState {}

class LocationSearchLoadedState extends LocationSearchState {
  final List<Address> locations;
  LocationSearchLoadedState(this.locations);
}

class LocationSearchErrorState extends LocationSearchState {
  final String error;

  LocationSearchErrorState(this.error);
}

class LocationSearchBloc
    extends Bloc<LocationSearchEvent, LocationSearchState> {
  final LocationRepository locationRepository;
  LocationSearchBloc(this.locationRepository)
      : super(LocationSearchIdleState());

  @override
  Stream<LocationSearchState> mapEventToState(
      LocationSearchEvent event) async* {
    if (event is LocationSearchLoadEvent) {
      yield LocationSearchLoadingState();
      AppResponse<List<Address>> response =
          await locationRepository.searchLocations(event.query);
      if (response.isSuccess) {
        yield LocationSearchLoadedState(response.data);
      } else {
        yield LocationSearchErrorState(response.error);
      }
    }
  }
}
