import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/ocg_location.dart';
import 'package:ocg_location/src/blocs/location_detail.dart';
import 'package:ocg_location/src/blocs/location_search.dart';
import 'package:ocg_location/src/models/address.dart';
import 'package:ocg_location/src/screens/location_picker_screen.dart';
import 'package:ocg_location/src/widgets/location_search_widget.dart';
import 'package:ocg_location/src/widgets/search_input_widget.dart';

class LocationSearchScreen extends StatefulWidget {
  final bool isMapEnabled;
  final bool showCurrentLocation;
  const LocationSearchScreen(
      {Key key, this.isMapEnabled = false, this.showCurrentLocation = false})
      : super(key: key);
  @override
  State<StatefulWidget> createState() => _LocationSearchScreenState();
}

class _LocationSearchScreenState extends State<LocationSearchScreen> {
  String query = '';
  TextEditingController controller;
  Address selected;

  final GlobalKey<ScaffoldState> _globalKey = GlobalKey<ScaffoldState>();

  @override
  void initState() {
    super.initState();
    controller = TextEditingController();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      if (widget.showCurrentLocation) {
        var state = BlocProvider.of<LocationCurrentBloc>(context).state;
        if (!(state is LocationCurrentLoadedState))
          _askCurrentLocation();
        else if (state is LocationCurrentLoadedState) {
          BlocProvider.of<LocationDetailBloc>(context)
              .add(LocationDetailPositionEvent(state.currentLocation));
        }
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _globalKey,
      appBar: AppBar(
        centerTitle: true,
        title: Text(
          "Location Search",
        ),
      ),
      body: BlocListener<LocationDetailBloc, LocationDetailState>(
        listener: (context, locationDetailState) {
          if (locationDetailState is LocationDetailLoadedState) {
            setState(() {
              selected = locationDetailState.location;
            });
          } else if (locationDetailState is LocationCurrentErrorState) {}
        },
        child: BlocListener<LocationCurrentBloc, LocationCurrentState>(
          listener: (context, state) {
            if (state is LocationCurrentLoadedState) {
              BlocProvider.of<LocationDetailBloc>(context)
                  .add(LocationDetailPositionEvent(state.currentLocation));
            } else if (state is LocationCurrentErrorState) {
              _globalKey.currentState
                  .showSnackBar(SnackBar(content: Text("${state.error}")));
            }
          },
          child: BlocBuilder<LocationSearchBloc, LocationSearchState>(
              builder: (context, locationSearchState) {
            return BlocBuilder<LocationDetailBloc, LocationDetailState>(
                builder: (context, locationDetailState) {
              return BlocBuilder<LocationCurrentBloc, LocationCurrentState>(
                  builder: (context, locationCurrentState) {
                return ModalProgressHUD(
                  inAsyncCall:
                      locationCurrentState is LocationCurrentLoadingState ||
                          locationDetailState is LocationDetailLoadingState,
                  child: SingleChildScrollView(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        SearchInputWidget(
                          onChanged: (val) {
                            setState(() {
                              this.query = val;
                            });
                          },
                          controller: controller,
                          label: "Search Location",
                        ),
                        ListTile(
                          leading: Icon(
                            selected?.fullAddress != null
                                ? Icons.location_on
                                : Icons.error,
                          ),
                          title: Text(
                            "${selected?.fullAddress ?? "Not Selected"}",
                          ),
                          onTap: null,
                        ),
                        Divider(),
                        widget.isMapEnabled
                            ? ListTile(
                                leading: Icon(
                                  Icons.location_on,
                                ),
                                title: Text(
                                  "Pick Location",
                                ),
                                trailing: Icon(
                                  Icons.chevron_right,
                                ),
                                onTap: () async {
                                  Address request = await Navigator.of(context)
                                      .push(
                                          MaterialPageRoute(builder: (context) {
                                    return LocationPickerScreen();
                                  }));
                                  if (request != null) {
                                    Navigator.of(context).pop(request);
                                  }
                                },
                              )
                            : Container(
                                width: 0,
                                height: 0,
                              ),
                        LocationSearch(
                          query: query,
                          onLocationSelected: (query) async {
                            BlocProvider.of<LocationDetailBloc>(context)
                                .add(LocationDetailPlaceEvent(query.placeId));
                          },
                        ),
                      ],
                    ),
                  ),
                );
              });
            });
          }),
        ),
      ),
      floatingActionButton: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 18),
        child: RoundedButton(
          "Continue",
          onTap: selected == null
              ? null
              : () {
                  Navigator.of(context).pop(selected);
                },
          borderRadius: 12,
          padding: EdgeInsets.symmetric(horizontal: 12, vertical: 16),
        ),
      ),
      floatingActionButtonLocation: FloatingActionButtonLocation.centerFloat,
    );
  }

  Future<void> _askCurrentLocation() async {
    BlocProvider.of<LocationCurrentBloc>(context)
        .add(LocationCurrentRefreshEvent());
  }
}
