import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_location/src/blocs/location_search.dart';
import 'package:ocg_location/src/models/address.dart';
import 'package:ocg_location/src/widgets/location_widget.dart';

class LocationSearch extends StatefulWidget {
  final String query;
  final Function(Address) onLocationSelected;
  const LocationSearch({Key key, this.query, this.onLocationSelected})
      : super(key: key);
  @override
  State<StatefulWidget> createState() => _LocationSearchState();
}

class _LocationSearchState extends State<LocationSearch> {
  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void didUpdateWidget(LocationSearch oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.query != oldWidget.query) {
      BlocProvider.of<LocationSearchBloc>(context)
          .add(LocationSearchLoadEvent(widget.query));
    }
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<LocationSearchBloc, LocationSearchState>(
      builder: (context, locationSearchState) {
        if (locationSearchState is LocationSearchLoadedState) {
          List<Address> cities = locationSearchState.locations;
          return ListView.builder(
            padding: EdgeInsets.all(0),
            itemBuilder: (context, index) {
              if (index == 0)
                return Padding(
                  padding: const EdgeInsets.only(top: 8.0),
                  child: ListTile(
                    title: Text("Search Results",
                        style: TextStyle(
                          fontWeight: FontWeight.w500,
                          fontSize: 18.0,
                        )),
                  ),
                );
              return LocationWidget(
                  title: "${cities[index - 1].area}",
                  onTap: () => widget.onLocationSelected(cities[index - 1]));
            },
            itemCount: cities.length + 1,
            shrinkWrap: true,
            physics: ClampingScrollPhysics(),
          );
        }
        if (locationSearchState is LocationSearchLoadingState)
          return Center(child: CircularProgressIndicator());
        return Container();
      },
    );
  }
}
